/**********************************************************************
 *
 * receptionist.h -- a web interface for the gsdl
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *********************************************************************/


#ifndef RECEPTIONIST_H
#define RECEPTIONIST_H

#include "gsdlconf.h"
#include "text_t.h"
#include "cgiargs.h"
#include "display.h"
#include "converter.h"
#include "cfgread.h"
#include "action.h"
#include "cnfgable.h"

// the MACROPRECEDENCE macro is used as a default. override
// it using something like:
//
// macroprecedence c,l
//
// in your configuration file
#ifndef MACROPRECEDENCE
#define MACROPRECEDENCE "l"
#endif

struct languageinfo_t {
  void clear();
  languageinfo_t () {clear();}

  text_t longname;
  text_t defaultencoding;
};

typedef map<text_t, languageinfo_t, lttext_t> languageinfo_tmap;
enum ldformat_t {LocalTime, UTCTime, Absolute};

struct recptconf {
   text_t gsdlhome;
   text_t gdbmhome;   // will equal gsdlhome if not set
   text_t httpprefix;
   text_t httpimg;    // will equal /images if not set
   text_t gwcgi;
   text_tset macrofiles;
   text_t saveconf;
   bool usecookies;  // true if we want cookies set
   bool logcgiargs;  // true if we want to log cgi arguments
   
   ldformat_t LogDateFormat;
   
   text_tmap pageparams;
   text_t macroprecedence;
   
   languageinfo_tmap languages;
   text_tmap encodings;
   
   void clear ();
   recptconf () {clear();}
};

class receptionist : public configurable {
protected:

   recptconf configinfo;
   
   cgiargsinfoclass argsinfo;
   actionmapclass actions;
   displayclass disp;
   convertinfoclass converters;
   
   virtual void prepare_page (action *a, cgiargsclass &args, 
			      outconvertclass &outconvert, 
	                      ostream &logout);
   
   bool get_cookie (text_t &cookie);
   bool get_cookie ();
   
   // get the default encoding for the given language - if it fails for any
   // reason, return ""
   text_t get_default_encoding (const text_t &language);
   
   virtual void define_general_macros (cgiargsclass &args, outconvertclass &outconvert, 
				       ostream &logout);
   
   bool append_logstr (const text_t &filename, const text_t &logstr,
		       ostream &logout);

   virtual void configure_encoding (const text_tarray &cfgline);
   
 public:
   receptionist ();
   virtual ~receptionist() {}
   
   // add_action makes another action available to the receptionist
   // the action remains the property of the calling code and that
   // code should destroy the action after the recptionist has been
   // destroyed.
   void add_action (action *theaction);
   actionmapclass *get_actionmap_ptr () {return &actions;}
   
   // add_converter makes another converter available to the receptionist.
   // Converters are needed to display pages using different encodings.
   // The converters remain the property of the calling code.
   void add_converter (const text_t &name, inconvertclass *inconverter,
		       rzwsoutconvertclass *outconverter) {
      converters.add_converter(name, inconverter, outconverter);}
   convertinfoclass *get_convertinfo_ptr () {return &converters;}
   
   
   // configure should be called for each line in the
  // configuration files to configure the receptionist and everything
  // it contains. The configuration should take place after everything
  // has been added but before the initialisation.
  void configure (const text_t &key, const text_tarray &cfgline);
  virtual void configure (const text_t &key, const text_t &value);
  const recptconf &get_configinfo () const {return configinfo;}
  cgiargsinfoclass *get_cgiargsinfo_ptr () {return &argsinfo;}
  
  virtual bool init (ostream &logout);
  
  virtual bool parse_cgi_args (const text_t &argstr, cgiargsclass &args, 
			       ostream &logout);

  virtual bool log_cgi_args (cgiargsclass &args, ostream &logout);

  text_t expandmacros (const text_t &astring, cgiargsclass &args, 
		       ostream &logout);

  // produce_cgi_page will call get_cgihead_info and 
  // produce_content in the appropriate way to output a cgi header and
  // the page content (if needed). If a page could not be created it
  // will return false
  virtual bool produce_cgi_page (cgiargsclass &args, ostream &contentout, 
				 ostream &logout);
  
  // get_cgihead_info determines the cgi header information for
  // a set of cgi arguments. If response contains location then
  // response_data contains the redirect address. If reponse
  // contains content then reponse_data contains the content-type.
  // Note that images can now be produced by the receptionist.
  virtual void get_cgihead_info (cgiargsclass &args, response_t &response,
				 text_t &response_data, ostream &logout);
  
  // produce the page content
  virtual bool produce_content (cgiargsclass &args, ostream &contentout,
				ostream &logout);
  
  // returns the compressed argument ("e") corresponding to the argument
  // list. This can be used to save preferences between sessions.
  virtual text_t get_compressed_arg (cgiargsclass &args, ostream &logout);
  
protected:
  // will read in all the macro files. If one is not found an
  // error message will be written to logout and the method will
  // return false.
  virtual bool read_macrofiles (ostream &logout);
  
};


#endif
